/**
 * 
 */
package testframework;

import java.io.File;

import com.gargoylesoftware.htmlunit.*;
import com.gargoylesoftware.htmlunit.html.*;

/**
 * @author lzcarl
 * A Wrapper for HtmlUnit
 */
public class HtmlUnitWrapper {
	
	private WebClient webClient;
	private WebWindow win;
	private HtmlPage currentPage;
	
	public HtmlUnitWrapper() {
		// use the default GUI-less browser
		this.webClient = new WebClient();
	}
	
	private HtmlPage getCurrentPage() throws Exception{
		win = webClient.getCurrentWindow();
		Page page = win.getEnclosedPage();
        if (page instanceof HtmlPage) {
        	currentPage = (HtmlPage) page;
        	return currentPage;
        }
        else
        	throw new RuntimeException("Non HTML content");
	}
	
	public void navigateTo(String url) throws Exception {
		currentPage = webClient.getPage(url);
		//System.out.println(currentPage.asXml());
		//getCurrentPage();
	}

	public void clickButton(String path) throws Exception {
		//return the first element
		HtmlButton button = (HtmlButton) ResourceLocator.locate(currentPage, path);
		currentPage = button.click();
	}
	public void setText(String path, String value) throws Exception{
		//return the first element
		HtmlTextInput input = (HtmlTextInput) ResourceLocator.locate(currentPage, path);
		input.setValueAttribute(value);
	}

	public String getInnerHtml(String path) throws Exception{
		HtmlElement element = ResourceLocator.locate(currentPage, path);
		return element.getTextContent();
	}
	
	public void typeInput(String path, char c) throws Exception{
		HtmlTextInput input = (HtmlTextInput) ResourceLocator.locate(currentPage, path);
		input.type(c);
	}	

	/**
	 * Type in a string gradually
	 * @param xpath
	 * @param str
	 * @throws Exception
	 */
	public void seqTypeInput(String path, String str) throws Exception{
		HtmlTextInput input = (HtmlTextInput) ResourceLocator.locate(currentPage, path);
		int i = 0;
		for(i = 0; i < str.length(); i ++) {
			input.type(str.charAt(i));
			Thread.sleep(200);
		}
	}
	
	public String getInputValue(String path) throws Exception {
		HtmlTextInput input = (HtmlTextInput) ResourceLocator.locate(currentPage, path);
		return input.getValueAttribute();
	}
	
	public int getListSize(String path) throws Exception {
		HtmlSelect input = (HtmlSelect) ResourceLocator.locate(currentPage, path);
		return input.getOptionSize();
	}
	
	public void setListOption(String path, int index) throws Exception {
		HtmlSelect input = (HtmlSelect) ResourceLocator.locate(currentPage, path);
		HtmlOption option = input.getOption(index);
		input.setSelectedAttribute(option, true);
	}
	
	public String getListOption(String path, int index) throws Exception {
		HtmlSelect input = (HtmlSelect) ResourceLocator.locate(currentPage, path);
		HtmlOption option = input.getOption(index);
		return option.asText();
	}
	
	public boolean getlistDisabled(String path) throws Exception {
		HtmlSelect input = (HtmlSelect) ResourceLocator.locate(currentPage, path);
		int size = input.getOptionSize();
		return (input.isDisabled() || (size == 0));
	}
	
	public void clickLink(String path) throws Exception {
		HtmlAnchor input = (HtmlAnchor) ResourceLocator.locate(currentPage, path);
		input.click();
	}
	
	public int getTableSize(String path) throws Exception {
		HtmlTable input = (HtmlTable) ResourceLocator.locate(currentPage, path);
		return input.getRowCount();
	}
	
	public void clickTableCell(String path, int index) throws Exception {
		HtmlTable input = (HtmlTable) ResourceLocator.locate(currentPage, path);
		HtmlTableCell cell = input.getRow(index).getCell(0);
		cell.click();
	}
	
	public String getTableCellText(String path, int index) throws Exception {
		HtmlTable input = (HtmlTable) ResourceLocator.locate(currentPage, path);
		HtmlTableCell cell = input.getRow(index).getCell(0);
		return cell.asText();
	}
	
	public boolean getControlExist(String path) {
		try {
			HtmlElement e = (HtmlElement) ResourceLocator.locate(currentPage, path);
			return true;
		} catch (Exception e) {
			// TODO Auto-generated catch block
			return false;
		}
	}
	
	public String getXML() {
		return currentPage.asXml();
	}
	
	public String getAttr(String path, String attr) throws Exception {
		HtmlElement e = (HtmlElement) ResourceLocator.locate(currentPage, path);
		return e.getAttribute(attr);
	}
	
	public void saveImage(String path, String filename) throws Exception {
		String src = getAttr(path, "src");
		System.out.println("Get img " + src);
		HtmlImage e = (HtmlImage) ResourceLocator.locate(currentPage, path);
		File f = new File(filename);
		e.saveAs(f);
	}
}
