/**
 * 
 */
package testframework;

import static org.junit.Assert.*;

import org.junit.*;
import java.io.*;

import traffic.*;
import testframework.utils.*;
import testframework.ui.*;

import java.util.*;
import java.text.SimpleDateFormat;

/**
 * @author lzcarl
 *
 */
public class WebTestCase {

	private HtmlUnitWrapper htmlUnit;
	private TestResult result = new TestResult();
	private String baseURL = "";
	private String interface_filename = "rerun.xml";
	private ArrayList<String> result_set;
	private CommandDriver driver = null;
	private String path = "";
	private ControlTree trafficTree;
	private boolean round;
	private boolean skipChildren = false;
	private int count = 0;
	private String prefix = "";
	private boolean countin = true;
	private SidebusterDemo demo;
	private int input_count;
	private double initial_entropy;
	/**
	 * Whether test the children of the parent nodes with empty traffic
	 */
	private String testEmptyParent;
	private String logfile = "";
	
	/**
	 * @throws java.lang.Exception
	 */
	@Before
	public void setUp() throws Exception {
		htmlUnit = new HtmlUnitWrapper();
		ConfigModule.load("network.properties");
		baseURL = ConfigModule.getProperty("baseURL");
		path = ConfigModule.getProperty("path");
		String round_str = ConfigModule.getProperty("round");
		String proto = ConfigModule.getProperty("proto");
		if(proto.equals("http") && round_str.equals("true"))
			round = true;
		else
			round = false;
		testEmptyParent = ConfigModule.getProperty("testEmptyParent");
		logfile = ConfigModule.getProperty("log");
		input_count = Integer.parseInt(ConfigModule.getProperty("item"));
	}

	private void startTrafficAnalyser() throws Exception{
		//get the parameters
		TrafficAnalyser.clientIP = ConfigModule.getProperty("clientIP");
		TrafficAnalyser.NIC = ConfigModule.getProperty("NIC");
		TrafficAnalyser.proto = ConfigModule.getProperty("proto");
		TrafficAnalyser.result = result;
		TrafficAnalyser.defaultStart();
	}
	
	private boolean process_one_cmd(Command cmd) throws Exception {
		//untestable unit, directly skip
		if(ControlUtils.disabled(cmd))
			return false;
		if(cmd.type.equals("TextBox") && cmd.action.equals("onKeyPress")) {
			if(!testEmptyParent.equals("true") && !trafficTree.isChild(cmd.index)) {
				skipChildren = true;
				return false;
			}
			String input = ControlUtils.getTextBoxValue(cmd.index);
			if(cmd.hierarchy.equals("self")) {
				String initial = "";
				int word_length = Integer.parseInt(cmd.count);
				//at the leaf level, add an additional feature: the upper nodes
				if(input.length() == word_length) {
					initial = input.substring(0, 2);
					prefix = "Group" + initial + ":";
				}else {
					countin = false;
				}
			}
			Tracer.trace("Set textbox " + cmd.name + "with value " + input);
			String key = cmd.name + cmd.action;
			TestResult.changeKey(key);
			htmlUnit.seqTypeInput(cmd.name, input);
			if(!TestResult.isEntryEmpty(key))
				trafficTree.add(cmd.index);
			if(cmd.time.equals(""))
				Thread.sleep(1000);
			else
				Thread.sleep(Integer.parseInt(cmd.time));
			return true;
		} else if(cmd.type.equals("Table") && cmd.action.equals("onSelection")) {
			String text = htmlUnit.getTableCellText(cmd.name, cmd.index);
			Tracer.trace("Choose Table " + cmd.name + " cell " + text + " with option " + cmd.index);
			TestResult.changeKey(cmd.name + cmd.action);
			htmlUnit.clickTableCell(cmd.name, cmd.index);
			if(cmd.time.equals(""))
				Thread.sleep(1000);
			else
				Thread.sleep(Integer.parseInt(cmd.time));
			return true;
		} else if (cmd.type.equals("Button") && cmd.action.equals("Click")) {
			Tracer.trace("Click button " + cmd.name);
			TestResult.changeKey(cmd.name + cmd.action);
			htmlUnit.clickButton(cmd.name);
			if(cmd.time.equals(""))
				Thread.sleep(1000);
			else
				Thread.sleep(Integer.parseInt(cmd.time));
			return true;
		} else if (cmd.type.equals("ListBox") && cmd.action.equals("Change")) {
			String option = htmlUnit.getListOption(cmd.name, cmd.index);
			Tracer.trace("Set listbox " + cmd.name + " with option " + option);
			TestResult.changeKey(cmd.name + cmd.action);
			htmlUnit.setListOption(cmd.name, cmd.index);
			if(cmd.time.equals(""))
				Thread.sleep(1000);
			else
				Thread.sleep(Integer.parseInt(cmd.time));
			return true;
		} else if (cmd.type.equals("HyperLink") && cmd.action.equals("Click")) {
			Tracer.trace("Click Link " + cmd.name);
			TestResult.changeKey(cmd.name + cmd.action);
			htmlUnit.clickLink(cmd.name);
			Thread.sleep(2000);
			return true;
		} else {
			throw new Exception("Unknown command");
		}
	}
	
	/**
	 * Parse the interface xml and execute related command
	 * @throws Exception
	 */
	private void process() throws Exception{
		ControlUtils.htmlUnit = htmlUnit;
		InterfaceParser parser = new InterfaceParser(interface_filename);
		ArrayList<Command> cmd_list = parser.parse();
		boolean status;
		Tracer.trace("Start to visit " + baseURL);
		htmlUnit.navigateTo(baseURL);
		Thread.sleep(10000);
		driver = new CommandDriver(cmd_list);
		driver.htmlUnit = htmlUnit;
		do {	
			clearControls(cmd_list);
			TestResult.clear();
			skipChildren = false;
			countin = true;
			prefix = "";
			for(Command cmd : cmd_list) {
				//System.out.println("Command " + cmd.action);
				status = process_one_cmd(cmd);
				if(skipChildren)
					break;
			}
			if(countin) {
				//result_set.add(prefix + TestResult.getResult());
				result_set.set(count ++ , prefix + TestResult.getResult());
				updateEntropy(false, count);
			}
		} while((cmd_list = driver.next()) != null);
		Thread.sleep(1000);
		Tracer.trace("All commands are executed");
	}
	
	private void clearControls(ArrayList<Command> cmd_list) throws Exception{
		for(Command cmd : cmd_list) {
			if(cmd.type.equals("TextBox")) {
				htmlUnit.setText(cmd.name, "");
				Thread.sleep(100);
			}
		}
	}
	
	private void save(String filename) throws Exception{
		BufferedWriter bufferedWriter = new BufferedWriter(new FileWriter(filename));
		for(int i = 0; i < result_set.size(); i ++) {
			bufferedWriter.write(result_set.get(i));
			bufferedWriter.newLine();
		}
		bufferedWriter.flush();
		bufferedWriter.close();
	}
	
	private void getRound(int base) {
		ArrayList<String> result_set2 = (ArrayList<String>) result_set.clone();
		InfoQuantifier.round_results(result_set2 ,base);
		/*System.out.println("The entropy of rounded result is " + 
				InfoQuantifier.get_entropy(result_set2) + "for base " + base);*/
		System.out.println("The conditional entropy of rounded result is " +
				InfoQuantifier.get_cond_entropy(result_set2) + "for base " + base);
	}
	
	private void init_result_set() {
		result_set = new ArrayList<String>();
		for(int i = 0; i < input_count; i ++) {
			result_set.add("untested");
		}
	}
	
	public void updateEntropy(boolean first, int count) {
		double entropy = InfoQuantifier.get_cond_entropy(result_set);
		//demo.clearGraphData();
		//demo.addGraphData(entropy, count);
		demo.addGraphData(entropy);
		demo.updateGraph();
		if(first)
			initial_entropy = entropy;
	}
	
	@Test
	public void test() {
		try {
			demo = new SidebusterDemo();
			//demo.adjustFrameSize();
			Tracer.demo = demo;
			ResourceLocator.method = path;
			String[] tags = new String[1];
			tags[0] = "input";
			ResourceLocator.initAcceptedTags(tags);
			demo.setUrl(baseURL);
			init_result_set();
			updateEntropy(true, 0);
			//block for user's action
			while(true) {
				if(demo.start)
					break;
				Thread.sleep(100);
			}
			trafficTree = new ControlTree();
			startTrafficAnalyser();
			process();
			TrafficAnalyser.stop();
			//TestResult.removeEmptyEntries();
			save(logfile);
			try{
				double entropy = InfoQuantifier.get_cond_entropy(result_set);
				Tracer.trace("The number of input is " + result_set.size());
				/*System.out.println("The entropy of the result is " + 
						InfoQuantifier.get_entropy(result_set));*/
				Tracer.trace("The conditional entropy of result is" + entropy);
				System.out.println("Save result to " + logfile);
				demo.showResult("The entropy before the test is " + initial_entropy + 
						"\n The entropy after the test is " + entropy);
				/*demo.showResult("The entropy before the test is 4.9\n" +
						"The entropy after the test is 1.3");*/
				//block for user's action
				while(true) {
					if(demo.end)
						break;
					Thread.sleep(100);
				}
				if(round) {
					for(int i = 2; i <= 512; i = i *2) {
						getRound(i);
					}
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}
