package traffic;

import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.util.Vector;

/**
 * 
 */

/**
 * @author wang63
 *
 */
public class Calculator {
	
	public static final DateFormat df = new SimpleDateFormat("hh:mm:ss.SSS");
	private static final long MAX_INTERVAL = 2000;
	public class Packet { //a simple definition of packet
		int groupID;  //group ID
		long time;
		String src;
		String dst;
		int len;
		
		public String toString() {
			return groupID+" - "+df.format(time)+" "+src+"->"+dst+"/"+len;
		}
	}
	
	Vector<Packet> packets = new Vector<Packet>();
	private String srcIP = "";

	public Calculator(String fileName, String srcIP) throws NumberFormatException, IOException, ParseException {
		FileInputStream fstream = new FileInputStream(fileName);
		DataInputStream in = new DataInputStream(fstream);
		BufferedReader br = new BufferedReader(new InputStreamReader(in));
		String strLine;
		this.srcIP = srcIP;
		long priorTime = 0;
		int groupID = 0;
		while ((strLine = br.readLine()) != null) {
			strLine = strLine.trim();
			if (strLine.isEmpty()) continue; //ignore empty line
			if (strLine.charAt(0) == '#') continue; //ignore comment line
			
			StringTokenizer st = new StringTokenizer(strLine); //cut into several tokens
			int k = 0;
			Packet packet = new Packet();
			while (st.hasMoreTokens()) {
				String token = st.nextToken();
				
				if (k == 0) {
					packet.time = df.parse(token).getTime();
				} else if (k == 1) { //event #
					int index = token.indexOf("->");
					packet.src = token.substring(0, index);
					packet.dst = token.substring(index+2,token.indexOf("("));
				} else if (k == 2) { //event type
					int fromIndex = 0;
					int endIndex = 0;
					while(true) {
						fromIndex = token.indexOf("/", fromIndex);
						if (fromIndex < 0) break;
						endIndex = token.indexOf(";", fromIndex);
						packet.len += Integer.valueOf(token.substring(fromIndex+1, endIndex));
						fromIndex = endIndex;
					}
				}
				
				k++;
			}
			
			packets.add(packet);
			
			if (priorTime == 0) {
				priorTime = packet.time;
			} else {
				if (Math.abs(packet.time-priorTime) > MAX_INTERVAL) {
					groupID++;
					System.out.println();
				}
				priorTime = packet.time;
			}
			packet.groupID = groupID;
			System.out.println(packet.toString());
		}
	}

	/**
	 * @param args
	 * @throws Exception 
	 */
	public static void main(String[] args) throws Exception {
//		int[] max = {499};
//		int[] min = {461};
//		int n = 0;
//		for (int k=4;k<=13;k++) {
//			int base = (int) Math.pow(2, k);
//			System.out.println(base+" bytes");
//			for (int i=0;i<min.length;i++) {
//				int m = min[i] % base;
//				if (m > 0) {
//					n = (max[i]-min[i]+1-m) % base;
//					if (n > 0) {
//						n = (max[i]-min[i]+1-base+m) / base + 2;
//					} else {
//						n = (max[i]-min[i]+1-base+m) / base + 1;
//					}
//				} else {
//					n = (max[i]-min[i]+1) % base;
//					if (n > 0) {
//						n = (max[i]-min[i]+1) / base + 1;
//					} else {
//						n = (max[i]-min[i]+1) / base;
//					}
//				}
//				System.out.println(n);
//			}
//		}
		
		//Calculator cal = new Calculator(args[0],args[1]);

		//cal.packetNumberForEachGroup();
		//System.out.println("state hiding: "+convertToPercentage(cal.stateHidingCost()));
//		for (int i=4;i<=11;i++) {
//			int k = (int) Math.pow(2, i);
//			System.out.println("round to "+k+" bytes:"+convertToPercentage(cal.roundPaddingCost(k)));
//		}
		//System.out.println("perfect padding: "+convertToPercentage(cal.perfectPaddingCost()));
		
		//reduction power of padding against hierarchical clicks
		Vector<Item> items = new Vector<Item>();
		items.add(new Item("A",257,1292,1190,5856,0));
		items.add(new Item("B",139,1300,1183,3818,1));
		items.add(new Item("C",251,1298,1181,5855,2));
		items.add(new Item("D",118,1305,1185,2839,3));
		items.add(new Item("E",120,1302,1183,2821,4));
		items.add(new Item("F",83,1287,1187,2049,5));
		items.add(new Item("G",93,1304,1182,2225,6));
		items.add(new Item("H",189,1325,1185,4248,7));
		items.add(new Item("I",91,1308,1184,2340,8));
		items.add(new Item("J",9,1301,1189,435,9));
		items.add(new Item("K",26,1291,1181,700,10));
		items.add(new Item("L",112,1300,1185,2689,11));
		items.add(new Item("M",169,1306,1187,3950,12));
		items.add(new Item("N",81,1304,1181,2141,13));
		items.add(new Item("O",63,1300,1187,1715,14));
		items.add(new Item("P",268,1315,1185,6120,15));
		items.add(new Item("Q",3,1286,1189,272,16));
		items.add(new Item("R",91,1308,1183,2381,17));
		items.add(new Item("S",228,1329,1184,5158,18));
		items.add(new Item("T",142,1304,1180,3288,19));
		items.add(new Item("U",42,1299,1185,1152,20));
		items.add(new Item("V",63,1294,1183,1647,21));
		items.add(new Item("W",21,1303,1182,730,22));
		items.add(new Item("X",3,1287,1185,268,23));
		items.add(new Item("Y",5,1269,1188,297,24));
		items.add(new Item("Z",2,1291,1270,260,25));
		items.add(new Item("0...9",1,1291,1291,226,26));

		int reductPower = reductionPowerOfPaddingForHierarchicalClicks(items,1);
		System.out.println(reductPower);
		for (int i=4;i<=9;i++) {
			int k = (int) Math.pow(2, i);
			System.out.println(""+i+"-byte padding");
			reductPower = reductionPowerOfPaddingForHierarchicalClicks(items,k);
			System.out.println(reductPower);
			System.out.println();
		}
	}

	private double stateHidingCost() throws Exception {
		int reqSizeMax = requestMaxLen();
		int resSizeMax = responseMaxLen();
		int reqNumMax = requestMaxNum();
		int resNumMax = responseMaxNum();
		int intMax = interactionMaxNum();
		int numOfGroup = (packets.get(packets.size()-1)).groupID+1;
		int ttlSize = totalSize();
		
		System.out.println("reqSizeMax="+reqSizeMax+",resSizeMax="+resSizeMax+","+
				"reqNumMax="+reqNumMax+",resNumMax="+resNumMax+",intMax="+intMax+
				",numOfGroup="+numOfGroup+",ttlSize="+ttlSize);
		
		
		int fnlSize = (reqSizeMax*reqNumMax+resSizeMax*resNumMax)*intMax*numOfGroup;
		return (fnlSize-ttlSize)*1.0/ttlSize;
	}
	
	private static int reductionPowerOfPaddingForHierarchicalClicks(Vector<Item> items,int roundTo) {
		mergeItemsOfPaddingForHierarchicalClicks(items,roundTo);
		
		int reductPower = 0;
		for (Enumeration<Item> enumm = items.elements();enumm.hasMoreElements();) {
			Item item = enumm.nextElement();
			item.updateStates(roundTo);
			
			//System.out.print(item.states+"("+item.size+","+Item.roundTo(item.size,roundTo)+") ");
			System.out.print(item.states+" ");
			reductPower += item.states;
		}
		System.out.println();
		return reductPower;
	}

	private static void mergeItemsOfPaddingForHierarchicalClicks(Vector<Item> items,int roundTo) {
		int[] paddedSizes = new int[items.size()];
		int i = 0;
		for (Enumeration<Item> enumm = items.elements();enumm.hasMoreElements();) {
			Item item = enumm.nextElement();
			
			if (item.groupID == -1) {
				paddedSizes[i++] = -1;
			} else {
				paddedSizes[i++] = Item.roundTo(item.size,roundTo);
			}
		}
		
		for (i=0;i<paddedSizes.length;i++) {
			for (int k=0;k<paddedSizes.length;k++) {
				Item item1 = items.get(i);
				Item item2 = items.get(k);
				
				if (item1.groupID == -1) break;
				if (item2.groupID == -1) continue;
				if (i == k) continue;
				
				if (paddedSizes[i] == paddedSizes[k]) {
					if (item1.states == 0) continue;
					if (item2.states == 0) continue;
					
					int newGroupID = Math.min(item1.groupID, item2.groupID);
					item1.groupID = newGroupID;
					item2.groupID = newGroupID;
					
					//data in item2 are merged into item1
					item1.elements += item2.elements;
					item1.maxSize = Math.max(item1.maxSize, item2.maxSize);
					item1.minSize = Math.min(item1.minSize, item2.minSize);
					item1.updateStates(roundTo);
					
					//clean item2
					item2.states = 0;
					item2.groupID = -1;
				}
			}
		}
	}

	private int totalSize() {
		int size = 0;
		for (Enumeration<Packet> enumm=packets.elements();enumm.hasMoreElements();) {
			Packet pkt = enumm.nextElement();
			
			size += pkt.len;
		}
		
		return size;
	}

	private int interactionMaxNum() {
		int maxNum = 0, num = -1, priorGroup = -1, numOfInts = 0;
		for (Enumeration<Packet> enumm=packets.elements();enumm.hasMoreElements();) {
			Packet pkt = enumm.nextElement();
			
			if (priorGroup < 0) {
				priorGroup = pkt.groupID;
			} else if (priorGroup != pkt.groupID) {
				num = -1;
				if (maxNum < numOfInts) maxNum = numOfInts;
				numOfInts = 0;
				priorGroup = pkt.groupID;
			}
			
			if (pkt.src.compareTo(this.srcIP) == 0) { //pkt is a request
				num = 0;
			} else {
				if (num == 0) { //request has been encountered
					num = 1;
					numOfInts++;
				}
			}
		}
		if (maxNum < numOfInts) maxNum = numOfInts;
		
		return maxNum;
	}

	private int responseMaxNum() {
		int maxNum = 0, num = 0, priorGroup = -1;
		for (Enumeration<Packet> enumm=packets.elements();enumm.hasMoreElements();) {
			Packet pkt = enumm.nextElement();
			
			if (priorGroup < 0) {
				priorGroup = pkt.groupID;
			} else if (priorGroup != pkt.groupID) {
				num = 0;
				priorGroup = pkt.groupID;
			}
			
			if (pkt.dst.compareTo(this.srcIP) == 0) { //pkt is a response
				num++;
			} else {
				if (maxNum < num)
					maxNum = num;
				num = 0;
			}
		}
		return maxNum;
	}

	private int requestMaxNum() {
		int maxNum = 0, num = 0, priorGroup = -1;
		for (Enumeration<Packet> enumm=packets.elements();enumm.hasMoreElements();) {
			Packet pkt = enumm.nextElement();
			
			if (priorGroup < 0) {
				priorGroup = pkt.groupID;
			} else if (priorGroup != pkt.groupID) {
				num = 0;
				priorGroup = pkt.groupID;
			}
			
			if (pkt.src.compareTo(this.srcIP) == 0) { //pkt is a request
				num++;
			} else {
				if (maxNum < num)
					maxNum = num;
				num = 0;
			}
		}
		return maxNum;
	}

	private void packetNumberForEachGroup() throws Exception {
		int groupID = -1;
		String str = "";
		for (Enumeration<Packet> enumm=packets.elements();enumm.hasMoreElements();) {
			Packet pkt = enumm.nextElement();
			
			if (groupID < 0) {
				groupID = pkt.groupID;
				str += groupID + " ";
			}
			
			if (groupID != pkt.groupID) { // a new group
				System.out.println(str);
				groupID = pkt.groupID;
				str = groupID + " ";
			} 
			if (pkt.src.compareTo(this.srcIP) == 0) {
				str += "+";
			} else if (pkt.dst.compareTo(this.srcIP) == 0) {
				str += "-";
			} else {
				throw new Exception("the packet is neither request nor response. "+pkt.toString());
			} 
			
		}
		System.out.println(str);
	}

	/**
	 * @param value
	 * @return
	 * convert to percentage
	 */
	private static String convertToPercentage(double value) {
		return ""+(((int) (value*1000))*1.0/10)+"%";
	}

	/**
	 * @return
	 * calculate the cost for perfect padding
	 * @throws Exception 
	 */
	private double perfectPaddingCost() throws Exception {
		int reqMax = requestMaxLen();
		int resMax = responseMaxLen();
		
		int cost = 0;
		int totalLen = 0;
		for (Enumeration<Packet> enumm=packets.elements();enumm.hasMoreElements();) {
			Packet pkt = enumm.nextElement();
			
			int costPerPkt = 0;
			if (pkt.src.compareTo(this.srcIP) == 0) {
				costPerPkt = roundPadding(pkt,reqMax);
			} else if (pkt.dst.compareTo(this.srcIP) == 0) {
				costPerPkt = roundPadding(pkt,resMax);
			} else {
				throw new Exception("the packet is neither request nor response. "+pkt.toString());
			}
			
			cost += costPerPkt;
			totalLen += pkt.len;
			
			//System.out.println(pkt.toString()+",pad "+costPerPkt);
		}
		
		System.out.println("reqMax="+reqMax+",resMax="+resMax);
		return cost*1.0/totalLen;
	}

	private int responseMaxLen() {
		int maxLen = 0;
		for (Enumeration<Packet> enumm=packets.elements();enumm.hasMoreElements();) {
			Packet pkt = enumm.nextElement();
			
			if (pkt.dst.compareTo(this.srcIP) == 0) {
				maxLen = (maxLen <= pkt.len)? pkt.len:maxLen;
			}
		}
		return maxLen;
	}

	private int requestMaxLen() {
		int maxLen = 0;
		for (Enumeration<Packet> enumm=packets.elements();enumm.hasMoreElements();) {
			Packet pkt = enumm.nextElement();
			
			if (pkt.src.compareTo(this.srcIP) == 0) {
				maxLen = (maxLen <= pkt.len)? pkt.len:maxLen;
			}
		}
		return maxLen;
	}

	/**
	 * @param roundTo
	 * @return
	 * calculate the cost for round padding
	 */
	private double roundPaddingCost(int roundTo) {
		int cost = 0;
		int totalLen = 0;
		for (Enumeration<Packet> enumm=packets.elements();enumm.hasMoreElements();) {
			Packet pkt = enumm.nextElement();
			
			int costPerPkt = roundPadding(pkt,roundTo);
			cost += costPerPkt;
			totalLen += pkt.len;
			
			//System.out.println(pkt.toString()+",pad "+costPerPkt);
		}
		
		return cost*1.0/totalLen;
	}

	/**
	 * @param pkt
	 * @param roundTo
	 * @return
	 * conduct round padding on pkt
	 */
	private int roundPadding(Packet pkt, int roundTo) {
		int mod = pkt.len % roundTo;
		if (mod == 0)
			return 0;
		else
			return roundTo - mod;
	}

}
