/**
 * 
 */

package traffic;

import java.io.UnsupportedEncodingException;
import java.sql.Time;

import traffic.utils.StringUtils;

import jpcap.packet.IPPacket;
import jpcap.packet.Packet;
import jpcap.packet.TCPPacket;

/**
 * @author t-ruwang
 *
 */
public class MyPacket {
	public enum ExplicitFeature {PacketSize /* packet size */, //observable features of a packet for attackers
		PacketNum/* packet number */,
		Timing_Gap /* timing represented by gaps between packets */, 
		Traffic_Bin /* group traffic to be multiple bins */ }

	private static final String TLS_HEADER = "170301";  //TLS packet header
	
	public static String clientIP = ""; //client IP
	public Packet packet = null;
	private static int packetLengthInBin = 0; //packet length in bin
	private static double priorTime = 0; //the arrival time of the prior packet

	public MyPacket(Packet packet) {
		this.packet = packet;
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// TODO Auto-generated method stub

	}

	public boolean isSSL(MyPacket pkt) {
		//to do: to detect if a packet is SSL, we need to detect SSL handshaking packets
		return true;
	}

	/* 
	 * toString()
	 * print src IP, dst IP, and first 5 bytes of payload
	 */
	public String toString() {
		int payloadLen = packet.data.length;
		IPPacket ippkt = ((IPPacket) packet);
		String data = "";
		int len = 5; //print the first 5 bytes of the payload
		if (packet.data.length >= len) {
			byte[] bytes = new byte[len];
			for (int i=0;i<bytes.length;i++) bytes[i] = packet.data[i];
			try {
				data = StringUtils.getHexString(bytes);
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
				System.exit(0);
			}
		}
		return getArrivalTimeAsString()+" "+ippkt.src_ip.getHostAddress()+"->"+ippkt.dst_ip.getHostAddress()+"("+String.format("%6d", payloadLen)+"B):"+data;
	}

	public int getHttpPacketLength() {
		return packet.data.length;
	}
	
	/**
	 * @return
	 * A packet with the first three bytes of its payload as "0x170301" is TLSv1 packet
	 * @throws UnsupportedEncodingException 
	 */
	public boolean isSSLDataPacket() throws UnsupportedEncodingException {
		String str = traffic.utils.StringUtils.getHexString(packet.data);
		if (str.contains(TLS_HEADER)) 
			return true;
		else
			return false;
	}

	/**
	 * @param bytes
	 * @return
	 * see if the first few bytes are matched by string
	 * @throws UnsupportedEncodingException 
	 */
	public boolean matchByString(String bytes) throws UnsupportedEncodingException {
		if ((bytes.length() % 2 == 1) && bytes.charAt(0)=='-') { //from server to client
			if (((IPPacket) packet).dst_ip.getHostAddress().compareTo(clientIP) != 0) {
				return false;
			}
			return matchByString(packet.data,bytes.substring(1),(bytes.length()-1)/2);
		} else {
			if (((IPPacket) packet).src_ip.getHostAddress().compareTo(clientIP) != 0) {
				return false;
			}
			return matchByString(packet.data,bytes,bytes.length()/2);
		}
	}

	/**
	 * @param data
	 * @param str
	 * @param len
	 * @return
	 * see if the first len bytes in data are matched with str
	 * @throws UnsupportedEncodingException 
	 */
	private boolean matchByString(byte[] data, String str, int len) throws UnsupportedEncodingException {
		if (data.length < len) return false;
		
		byte[] bytes = new byte[len];
		for (int i=0;i<bytes.length;i++) bytes[i] = data[i];
		if (str.compareTo(StringUtils.getHexString(bytes)) == 0) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * @param explicitFeature
	 * @param param
	 * @return
	 * Visualize the packet with textual presentation
	 */
	public String visualize(ExplicitFeature explicitFeature,double param) {
		String str = "";
		switch (explicitFeature) {
		case PacketSize: { //visualize packet size
			for (int i=0;i<packet.data.length/10;i++) {
				str += "-";
			}
			str += packet.data.length;
			break;
		}
		case Timing_Gap: {//visualize timing
			double time = getArrivalTime();
			//time = (int) ((((time/1000) % 1) * 1000000) / 1) * 1.0 / 1000;
			double gap = ((int) ((time-priorTime)*1000)) * 1.0 / 1000; 
			//str += gap + " " + packet.data.length;
			str += textualPresentation(gap,param) + gap;
			priorTime = time;
			break;
		}
		case PacketNum: { //visualize packet num
			//todo: packet size visualization
			break;
		}
		case Traffic_Bin: { //visualize data length for each bin 
			double time = getArrivalTime();
			double gap = time-priorTime;
			//System.out.println("gap="+gap+",param="+param);
			if (gap <= param) {
				packetLengthInBin  += packet.data.length;
			} else {
				packetLengthInBin = packet.data.length;
				priorTime = time;
			}
			str += textualPresentation(packetLengthInBin,10) + packetLengthInBin;
			break;
		}
		default:
			System.err.println("unsupported explicit feature "+explicitFeature);
			System.exit(0);
		}
		
		return str;
	}
	
	/**
	 * @param num
	 * @param precision
	 * @return
	 * return the textual presentation for num
	 */
	private String textualPresentation(double num, double precision) {
		String str = "";
		int count = (int) (num / precision);
		int limit = 100;
		if (count > limit) count = limit; //limit
		for (int i=0;i<count;i++) {
			str += "-";
		}
		return str;
	}

	/**
	 * @param packet
	 * @return
	 * generate arrival time for packet
	 */
	public double getArrivalTime() {
		return packet.sec+packet.usec*1e-6;
	}
	
	/**
	 * @return
	 * get arrival time in the form of string
	 * the format of the output is "HH:MM:SS.mmm". 
	 * an example is "15:26:36.355"
	 */
	public String getArrivalTimeAsString() {
		return (new Time(packet.sec*1000)).toString()+"."+Integer.toString((int) packet.usec/1000);
	}

	//Added by lz
	//only record the packet size
	public String toSimplifiedPacketString() throws UnsupportedEncodingException {
		int payloadLen = packet.data.length;
		IPPacket ippkt = ((IPPacket) packet);		
		String data = traffic.utils.StringUtils.getHexString(packet.data);
		int len = 5; //print the first 5 bytes of the payload
		int fromIndex = 0,headIndex = 0;
		String str = "";
		while ((headIndex=data.indexOf(TLS_HEADER, fromIndex)) >= 0) {
			str += Integer.valueOf(data.substring(headIndex+6, headIndex+10), 16)+";";
			fromIndex = headIndex + len*2;
		}
		/*String str = getArrivalTimeAsString()+" "+ippkt.src_ip.getHostAddress()+"->"+ippkt.dst_ip.getHostAddress()+"("+String.format("%6d", payloadLen)+"B):";
		while ((headIndex=data.indexOf(TLS_HEADER, fromIndex)) >= 0) {
			str += headIndex+"-"+data.substring(headIndex, headIndex+len*2)+"/"+Integer.valueOf(data.substring(headIndex+6, headIndex+10), 16)+";";
			fromIndex = headIndex + len*2;
		}*/
		return str;
	}
	
	public String toSSLDataPacketString() throws UnsupportedEncodingException {
		int payloadLen = packet.data.length;
		IPPacket ippkt = ((IPPacket) packet);
		String data = traffic.utils.StringUtils.getHexString(packet.data);
		int len = 5; //print the first 5 bytes of the payload
		int fromIndex = 0,headIndex = 0;
		String str = getArrivalTimeAsString()+" "+ippkt.src_ip.getHostAddress()+"->"+ippkt.dst_ip.getHostAddress()+"("+String.format("%6d", payloadLen)+"B):";
		while ((headIndex=data.indexOf(TLS_HEADER, fromIndex)) >= 0) {
			str += headIndex+"-"+data.substring(headIndex, headIndex+len*2)+"/"+Integer.valueOf(data.substring(headIndex+6, headIndex+10), 16)+";";
			fromIndex = headIndex + len*2;
		}
//		String str = "";
//		while ((headIndex=data.indexOf(TLS_HEADER, fromIndex)) >= 0) {
//			if (ippkt.src_ip.getHostAddress().compareTo(clientIP) == 0) str += "+";
//			else if (ippkt.dst_ip.getHostAddress().compareTo(clientIP) == 0) str += "-";
//			else str += "*";
//			str += Integer.valueOf(data.substring(headIndex+6, headIndex+10), 16);
//			fromIndex = headIndex + len*2;
//		}
		return str;
	}
}
