package traffic;

import java.io.IOException;
import java.io.UnsupportedEncodingException;

import jpcap.*;
import jpcap.packet.IPPacket;
import jpcap.packet.TCPPacket;
import jpcap.packet.Packet;

import testframework.TestResult;
import testframework.utils.*;

/**
 * 
 */
class Tcpdump implements PacketReceiver,Runnable {
	public enum TransactionStatus {
		InTransaction, //In transaction
		OutTransaction, //Out of transaction
		NoTransaction  //No transaction
	}
	
	public enum Protocol { //traffic protocol
		http, //http
		https, //https
		unknown //unknown
	}
	
	public static Protocol proto = Protocol.unknown; //sniff the traffic with specific protocol
	public static long longestWaitingTime = 1000; //longest waiting time is 1000 ms
	Thread t;
	JpcapCaptor jpcap = null;
	private long lastReceiveTime = System.currentTimeMillis() ; //the time when the last packet is received
	private long currentReceiveTime = System.currentTimeMillis(); ; //the time when the current packet is received
	public static TransactionStatus transStatus = TransactionStatus.NoTransaction; //transaction status

	public TestResult result;
	public String clientIP;
	
	public void receivePacket(Packet packet) {
		while (transStatus == TransactionStatus.InTransaction) ; //wait trans
		
		if (packet.data.length > 0) { //ignore empty packet
			//System.out.println("Capture packets");
			MyPacket pkt = new MyPacket(packet);
			currentReceiveTime = System.currentTimeMillis(); 
			if (currentReceiveTime-lastReceiveTime > longestWaitingTime) {
				System.out.println();
			}
			lastReceiveTime = System.currentTimeMillis();
			
			if (((TCPPacket) pkt.packet).src_port == 3389 || ((TCPPacket) pkt.packet).dst_port == 3389) return; //filter remote desktop traffic
			
			try {
				String direction = "";
				String src_ip = "";
				String traffic = "";
				String[] traffic_parts = null;
				int i = 0;
				String str = "";				
				switch (proto) {
				case http:
					System.out.println(pkt.toString());
					//added by lz
					//record the captured traffic
					direction = "";
					src_ip = ((TCPPacket) pkt.packet).src_ip.toString();
					//System.out.println("src_ip=" + src_ip);
					if (src_ip.contains(clientIP))
						direction = "<";
					else
						direction = ">";
					int size = pkt.getHttpPacketLength();
					str = direction + size;
					Tracer.updateTraffic(direction, String.valueOf(size));
					TestResult.updateEntry(str);
					//TestResult.test(str);
					//System.out.println(str);					
					break;
				case https:
					if (pkt.isSSLDataPacket()) {
						System.out.println(pkt.toSSLDataPacketString());
						//System.out.println(pkt.toSimplifiedPacketString());
						//added by lz
						//record the captured traffic
						direction = "";
						src_ip = ((TCPPacket) pkt.packet).src_ip.toString();
						//System.out.println("src_ip=" + src_ip);
						if (src_ip.contains(clientIP))
							direction = "<";
						else
							direction = ">";
						traffic = pkt.toSimplifiedPacketString();
						traffic_parts = traffic.split(";");
						i = 0;
						str = "";
						for(i = 0; i < traffic_parts.length; i ++) {
							str = str + direction + traffic_parts[i];
							Tracer.updateTraffic(direction, traffic_parts[i]);
						}
						TestResult.updateEntry(str);
						//TestResult.test(str);
						//System.out.println(str);
					}
					break;
				case unknown:
					System.out.println(pkt.toString());
					break;
				default:
					System.out.println(pkt.toString());
				}	
				 
			} catch (UnsupportedEncodingException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				System.exit(0);
			}
			//System.out.println(pkt.toString());
		}
		
		if (transStatus == TransactionStatus.InTransaction) transStatus = TransactionStatus.OutTransaction; //in trans -> out trans
	}
	
	public Tcpdump(int interfaceID, String filter, TransactionStatus status, String clientIP) throws IOException {
		NetworkInterface[] devices = JpcapCaptor.getDeviceList();
		
		jpcap = JpcapCaptor.openDevice(devices[interfaceID], 2000, false, 20);

		jpcap.setFilter(filter, false);  // set a filter
		
		transStatus = status;
		
		this.clientIP = clientIP; 
		
		t = new Thread(this,"My thread");
		t.start();
	}
	
	public void stop() {
		if(!(jpcap == null))
			jpcap.close();
	}
	
	public static void main(String[] args) throws Exception {
		if (args.length < 1) {
			NetworkInterface[] devices = JpcapCaptor.getDeviceList();
			System.out.println("usage: java Tcpdump <select a number from the following>");
			
			for (int i = 0; i < devices.length; i++) {
				System.out.println(i+" :"+devices[i].name + "(" + devices[i].description+")");
				System.out.println("    data link:"+devices[i].datalink_name + "("
						+ devices[i].datalink_description+")");
				System.out.print("    MAC address:");
				for (byte b : devices[i].mac_address)
					System.out.print(Integer.toHexString(b&0xff) + ":");
				System.out.println();
				for (NetworkInterfaceAddress a : devices[i].addresses)
					System.out.println("    address:"+a.address + " " + a.subnet + " "
							+ a.broadcast);
			}	
			System.exit(0);
		}
		
		Tcpdump tcpdump = new Tcpdump(Integer.valueOf(args[0]),"host 172.31.42.13",TransactionStatus.NoTransaction, "192.168.2.2");
	}

	@Override
	public void run() {
		jpcap.loopPacket(-1, this);
		//jpcap.processPacket(10,this);
	}
	
	/**
	 * @param protoStr
	 * @return
	 * obtain protocol by parsing protoStr
	 * @throws Exception 
	 */
	public static Protocol getProtocolFromString(String protoStr) throws Exception {
		if (protoStr.compareToIgnoreCase(Protocol.http.toString()) == 0) {
			return Protocol.http;
		} else if (protoStr.compareToIgnoreCase(Protocol.https.toString()) == 0) {
			return Protocol.https;
		} else if (protoStr.compareToIgnoreCase(Protocol.unknown.toString()) == 0) {
			return Protocol.unknown;
		} else {
			throw new Exception("unknown or invalid protocol name "+protoStr);
		}
	}
}

